<?php
namespace Teknisa\DF\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource;
use Teknisa\DF\Services\EmailsAdicionaisService;

class EmailsAdicionaisController{

    protected $service;
    
    public function __construct(EmailsAdicionaisService $service) {
        $this->service = $service;
    }
    
    public function findEmailsAdicionais(DTO\Request\Filter $request, DTO\Response $response){
        try {
            $filters = $request->getFilterCriteria()->getConditions();
            $page = $request->getFilterCriteria()->getPage();
            $pageSize = $request->getFilterCriteria()->getPageSize();
            $orderBy = $request->getFilterCriteria()->getOrderBy();
            
            $emails = $this->service->findEmailsAdicionais($filters, $page, $pageSize, $orderBy);
            $response->addDataSet(new DataSource\DataSet('findEmailsAdicionais', $emails));
        } catch (\Exception $ex){
            $response->setError(new Error($ex->getMessage(), $ex->getCode()));
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
        }
        
    }
    
    public function findTransportadoras(DTO\Request\Filter $request, DTO\Response $response){
        try {
            $filters = $request->getFilterCriteria()->getConditions();
            $page = $request->getFilterCriteria()->getPage();
            $pageSize = $request->getFilterCriteria()->getPageSize();
            $orderBy = $request->getFilterCriteria()->getOrderBy();
            $trasnportadoras = $this->service->findTransportadoras($filters, $page, $pageSize, $orderBy);
            $response->addDataSet(new DataSource\DataSet('findTransportadoras', $trasnportadoras));
        } catch (\Exception $ex){
            $response->setError(new Error($ex->getMessage(), $ex->getCode()));
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
        }
    }
    
    public function saveEmailsAdicionais(DTO\Request\Row $request, DTO\Response $response){
        $emailRow = $request->getRow();
        try {
            $result = $this->service->saveEmailsAdicionais($emailRow);
            $response->addDataSet(new DataSource\DataSet('data', array()));
        } catch(\Exception $ex) {
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }
    
    public function removeEmailAdicional(DTO\Request\Row $requisicao, DTO\Response $resposta){
        try {
            $email = $requisicao->getRow();
            $idEmail = $email['IDEMAIL'];
            $this->service->removeEmailAdicional($idEmail);
            $resposta->addMessage(new DTO\Response\Message("Item removido com sucesso."));
        } catch(\Exception $ex){
            $resposta->addMessage(new DTO\Response\Message($ex->getMessage()));
        }
    }
    
    
}